==> libfb-v0.18.3b-src.zip: src/rtlib/libfb_strw_instrrev.c <==
/*
 *  libfb - FreeBASIC's runtime library
 *	Copyright (C) 2004-2007 The FreeBASIC development team.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As a special exception, the copyright holders of this library give
 *  you permission to link this library with independent modules to
 *  produce an executable, regardless of the license terms of these
 *  independent modules, and to copy and distribute the resulting
 *  executable under terms of your choice, provided that you also meet,
 *  for each linked independent module, the terms and conditions of the
 *  license of that module. An independent module is a module which is
 *  not derived from or based on this library. If you modify this library,
 *  you may extend this exception to your version of the library, but
 *  you are not obligated to do so. If you do not wish to do so, delete
 *  this exception statement from your version.
 */

/*
 * strw_instrrev.c -- instrrevw function
 *
 * chng: dec/2007 written [jeffm]
 *
 */

#include "fb.h"

/*:::::*/
FBCALL int fb_WstrInstrRev ( const FB_WCHAR *src, const FB_WCHAR *patt, int start )
{
	if( (src != NULL) && (patt != NULL) )
	{
		size_t size_src = fb_wstr_Len(src);
		size_t size_patt = fb_wstr_Len(patt);
		size_t i, j;

		if( (size_src != 0) && (size_patt != 0) && (start >= 0) && (start <= size_src) )
		{

			/* If start is the default value then start from the end */
			if( (start == 0) || (start > size_src - size_patt) )
				start = size_src - size_patt + 1;
			
			/*
				There is no wcsrstr() function, 
				so instead do a brute force scan.
			*/

			for( i=0; i<start; ++i ) 
			{
				for( j=0; j!=size_patt; ++j ) 
				{
					if( src[start-i+j-1] != patt[j] )
						break;
				}
				if( j==size_patt )
					return start - i;
			}
		}
	}
		
	return 0;
}

==> libfb-v0.20.0b-src.zip: src/rtlib/libfb_strw_instrrev.c <==
/*
 *  libfb - FreeBASIC's runtime library
 *	Copyright (C) 2004-2008 The FreeBASIC development team.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  As a special exception, the copyright holders of this library give
 *  you permission to link this library with independent modules to
 *  produce an executable, regardless of the license terms of these
 *  independent modules, and to copy and distribute the resulting
 *  executable under terms of your choice, provided that you also meet,
 *  for each linked independent module, the terms and conditions of the
 *  license of that module. An independent module is a module which is
 *  not derived from or based on this library. If you modify this library,
 *  you may extend this exception to your version of the library, but
 *  you are not obligated to do so. If you do not wish to do so, delete
 *  this exception statement from your version.
 */

/*
 * strw_instrrev.c -- instrrevw function
 *
 * chng: dec/2007 written [jeffm]
 *
 */

#include "fb.h"

/*:::::*/
FBCALL int fb_WstrInstrRev ( const FB_WCHAR *src, const FB_WCHAR *patt, int start )
{
	if( (src != NULL) && (patt != NULL) )
	{
		size_t size_src = fb_wstr_Len(src);
		size_t size_patt = fb_wstr_Len(patt);
		size_t i, j;

		if( (size_src != 0) && (size_patt != 0) && (size_patt <= size_src) && (start != 0))
		{
			/* handle signed/unsigned comparisons of start and size_* vars */
			if( start < 0 )
				start = size_src - size_patt + 1;
			else if( start > size_src )
				start = 0;
			else if(start > size_src - size_patt)
				start = size_src - size_patt + 1;
			
			/*
				There is no wcsrstr() function, 
				so instead do a brute force scan.
			*/

			for( i=0; i<start; ++i ) 
			{
				for( j=0; j!=size_patt; ++j ) 
				{
					if( src[start-i+j-1] != patt[j] )
						break;
				}
				if( j==size_patt )
					return start - i;
			}
		}
	}
		
	return 0;
}
